from peewee import (
    Model,
    MySQLDatabase,
    AutoField,
    IntegerField,
    CharField,
    TextField,
    ForeignKeyField,
    DateTimeField,
    DoubleField,
    BooleanField,
)
import splogger as log
import datetime
import traceback
import sys, os

from flask import Flask, request, jsonify
from flask_cors import CORS
from time import sleep

class Raspberry(Model):
    id = AutoField()
    uid = CharField()
    last_seen = DateTimeField()
    last_ip = CharField()

CONNECTION = None

def create_mysql_connection() -> MySQLDatabase:
    global CONNECTION

    assert CONNECTION == None
    if not "MYSQL_HOST" in os.environ:
        os.environ["MYSQL_HOST"] = "127.0.0.1"
        os.environ["MYSQL_DB"] = "db"
        os.environ["MYSQL_USER"] = "user"
        os.environ["MYSQL_PASSWORD"] = "password"
        log.warning("No MYSQL_HOST, using default values")

    url = os.environ["MYSQL_HOST"]
    log.fine("Using MYSQL_HOST %s" % url)
    return MySQLDatabase(
        os.environ["MYSQL_DB"],
        user=os.environ["MYSQL_USER"],
        host = url,
        password=os.environ["MYSQL_PASSWORD"],
        port=3306,
    )


def init_db_connection():
    while True:
        try:
            db = create_mysql_connection()
            db.connect()
            tables = [Raspberry]  # , Printer, PrinterQueue]
            db.bind(tables)
            db.create_tables(tables)
        except:
            traceback.print_exc()
            print("Failed to connect, retrying")
            sleep(1)
            continue
        return


## Utils

def get_or_update_rasp(infos):
    req = Raspberry.select().where(Raspberry.uid == infos['rid'])
    if req.count():
        # update
        Raspberry.update({Raspberry.last_seen: datetime.datetime.now(), Raspberry.last_ip: infos['ip']}).where(Raspberry.uid == infos['rid']).execute()
    else:
        # create
        Raspberry.create(uid = infos['rid'], last_seen = datetime.datetime.now(), last_ip = infos['ip'])

### Flask part 

app = Flask(__name__)
CORS(app)

@app.route("/<rid>")
def rasp_identify(rid):
    get_or_update_rasp({
        'rid': rid,
        'ip': request.remote_addr
    })
    return jsonify({})

@app.route("/list")
def list_rasps():
    rasps = []
    for rasp in Raspberry.select().order_by(Raspberry.last_seen.desc()):
        rasps.append({
            'uid': rasp.uid,
            'ip': rasp.last_ip,
            'last_seen': rasp.last_seen
        })
    return jsonify(rasps)


if __name__ == "__main__":
    init_db_connection()
    app.run(port=3324, host="0.0.0.0")

# user = User.select().where(User.username == "test").count()
# print(user)
# print("fin")

# if user:
#     User.delete_by_id("test")
# #User.create(username="test", fullname="test TEST")
# user = User.select().where(User.username == "test").count()
# print(user)

# for user in User.select().where(User.username == "test"):
#     print(user)
# print("fin")
